<?php

/**
 * Your installation or use of this SugarCRM file is subject to the applicable
 * terms available at http://support.sugarcrm.com/Resources/Master_Subscription_Agreements/
 * except that this file is provided "as is" without warranty and no support
 * will be provided by SugarCRM for this file.
 *
 * If you do not agree to all of the applicable terms or do not have the
 * authority to bind the entity as an authorized representative, then do not
 * install or use this SugarCRM file.
 *
 * Copyright (C) SugarCRM Inc. All rights reserved.
 *
 * @package Scheduler-run
 * @author Olivier Nepomiachty
 * @version 0.4.1
 */

use Sugarcrm\Sugarcrm\ProcessManager\Registry;

class SchedulerRun extends SugarApi{

    // This function is only called whenever the rest service cache file is deleted.
    // This shoud return an array of arrays that define how different paths map to different functions
    public function registerApiRest() {
        return array(
            //GET
            'GetSchedulers' => array(
                //request type
                'reqType' => 'POST',

                //set authentication
                //'noLoginRequired' => false,

                //endpoint path
                'path' => array('SchedulerRun'),

                //endpoint variables
                'pathVars' => array(''),

                //method to call
                'method' => 'schedulerRunJob',

                //short help string to be displayed in the help documentation
                'shortHelp' => '',

                //long help to be displayed in the help documentation
                'longHelp' => '',
            ),
        );
    }
    
    private function runJob($job){
        
        $now = $job->db->now();
        $queued = SchedulersJob::JOB_STATUS_QUEUED;


        $job->status = SchedulersJob::JOB_STATUS_RUNNING;
        $job->client = 'mobile';

        $client = $job->db->quote('mobile');
        // using direct query here to be able to fetch affected count
        // if count is 0 this means somebody changed the job status and we have to try again

        $res = $job->db->query("UPDATE job_queue SET status='{$job->status}', date_modified=$now, client='$client' WHERE id='{$job->id}' AND status='$queued'");
        if($job->db->getAffectedRowCount($res) == 0) {
            // somebody stole our job, try again
            throw new SugarApiExceptionInvalidParameter('Someone stole the job before it was ran');
        } else {
            // to update dates & possible hooks
            $job->save();
        }

        /**
         * PHASE 2
         */

        // Before calling save, we need to clear out any existing registered AWF
        // triggered start events so they can continue to trigger.
        Registry\Registry::getInstance()->drop('triggered_starts');

        //$this->setTimeLimit($this->max_runtime);
        $res = $job->runJob();
        //$this->clearTimeLimit();
        if (!$res) {
            // if some job fails, change run status
            throw new Exception("Job failed!!!");
        }
        
    }
    
    function schedulerRunJob($api, $args)
    {

        $jobs_to_run = [];
        $scheduler_task = '';
        $other_scheduler_task = '';
        $specific_jobs = ['Dispatch Web Logic Hook', 'Run Advanced Workflows Definitions'];
        if (in_array($args['scheduler'], $specific_jobs)) {
            switch ($args['scheduler']) {
                case 'Dispatch Web Logic Hook' :
                    $other_scheduler_task = 'class::WebLogicHook';
                    break;
                case 'Run Advanced Workflows Definitions' :
                    $other_scheduler_task = 'function::PMSEJobRun';
                    break;
                default:
                    break;
            }

        } else {
            $scheduler_task = $args['scheduler'];
        }




        /**
         * If the user chooses Advanced Workflow Scheduled Job, execute ALL PMSE queued tasks,
         * as this is what users expect when running the WF scheduled job
         */
        if (!empty($other_scheduler_task)) {
            $ac = new Account();

            $sql = sprintf("select id from job_queue where target = '%s' and status = 'queued' and deleted = 0", $other_scheduler_task);
            $res = $ac->db->query($sql);
            if (!$res) {
                return array('Error' => $ac->db->lastDbError());
            }
            // TODO: Possibly "unhardcode" the parameter strings

            while ($row = $ac->db->fetchByAssoc($res)) {
                $job_id = $row['id'];
                $job = BeanFactory::getBean('SchedulersJobs', $job_id);
                $jobs_to_run[] = $job;
            }


            foreach ($jobs_to_run as $job) {
                $this->runJob($job);
            }
        }

        if (!empty($scheduler_task)) {
            $scheduler = BeanFactory::getBean("Schedulers", $scheduler_task);

            $job = $scheduler->createJob();

            $queue = new SugarJobQueue();
            $job_id = $queue->submitJob($job);

            $job = BeanFactory::getBean('SchedulersJobs', $job_id);


            $this->runJob($job);
        }

        return true;
    }
}
