/**
 * Your installation or use of this SugarCRM file is subject to the applicable
 * terms available at http://support.sugarcrm.com/Resources/Master_Subscription_Agreements/
 * except that this file is provided "as is" without warranty and no support
 * will be provided by SugarCRM for this file.
 *
 * If you do not agree to all of the applicable terms or do not have the
 * authority to bind the entity as an authorized representative, then do not
 * install or use this SugarCRM file.
 *
 * Copyright (C) SugarCRM Inc. All rights reserved.
 *
 * @package Scheduler-run
 * @author Olivier Nepomiachty
 * @version 0.4.1
 */
({
    plugins: ['Dashlet'],
    
    events: {
        "submit #scheduler-run-submit": "runSelectedScheduler"
    },
    
    initialize: function (options) {
        this.plugins = _.union(this.plugins || [], ['HistoricalSummary']);
        this._super('initialize', [options]);        
        this.schedulers = [];  
    },
    
    loadData: function () {
        
        var view = this;
        
        var schedulers = app.data.createBeanCollection('Schedulers').fetch({
            limit: -1,
            success: function (data) {
                view.schedulers = data.models;
                //add to scheduler list tasks Advanced Workflows definition and Web Logic Hooks
                var aw_tasks = app.data.createBean('Schedulers');
                    aw_tasks.set({id:'Run Advanced Workflows Definitions',name:'Run Advanced Workflows Definitions'});
                var web_logic_hooks = app.data.createBean('Schedulers');
                    web_logic_hooks.set({id:'Dispatch Web Logic Hook',name:'Dispatch Web Logic Hook'});
                view.schedulers.push(aw_tasks, web_logic_hooks);
                view.render();
            }
        });
                
    },
    
    _logEvent:function (message){
        
        var timestamp = moment().format('HH:mm:ss');
        this.$el.find('.logarea').append(timestamp+": "+message+"<br />");
        
//        var height = this.$el.find('.logarea').scrollHeight;
        this.$el.find('.logarea').scrollTop(1E10);
        
    },
    
    _buttonLock: function (){
        this.$el.find('button[type=submit]').attr('disabled', 'disabled');   
    },
    
    _buttonLockRelease: function (){
        this.$el.find('button[type=submit]').removeAttr('disabled');
    },
    
    runSelectedScheduler: function (e) {
        
        var view = this;
        
        e.preventDefault();
        
        var form_data = $(e.target).serializeArray();
        
        var params = {};
        _.each(form_data, function (i) {
            params[i.name] = i.value;
        });

        var name_selected = this.$el.find('form select > option:selected').text()

        view._logEvent("Request to run scheduler '"+name_selected+"'");
        this._buttonLock();
        
        

        var url = app.api.buildURL(null, 'SchedulerRun', params);
        
        var xhr = app.api.call('create', url, params, {
            success: function (data) {
                view._logEvent("Job run SUCESS\n");
                view._buttonLockRelease()
            },
            error: function (e){
                alert("Job FAILED");
                view._logEvent("Job run FAILURE!\n");
                view._buttonLockRelease();
            }
        });
       
    }
    
    
})
